#include "mirage_draw.h"
#include "mirage_style.h"
#include "mirage_types.h"

#include "support.h"

#include <cairo.h>

void
mirage_rounded_rectangle (cairo_t *cr,
                                 double x, double y, double w, double h,
                                 double radius, uint8 corners)
{
	if (corners & MIRAGE_CORNER_TOPLEFT)
		cairo_move_to (cr, x+radius, y);
	else
		cairo_move_to (cr, x, y);
	
	if (corners & MIRAGE_CORNER_TOPRIGHT)
		cairo_arc (cr, x+w-radius, y+radius, radius, M_PI + M_PI*0.5, M_PI*2);
	else
		cairo_line_to (cr, x+w, y);
	
	if (corners & MIRAGE_CORNER_BOTTOMRIGHT)
		cairo_arc (cr, x+w-radius, y+h-radius, radius, 0, M_PI/2);
	else
		cairo_line_to (cr, x+w, y+h);
	
	if (corners & MIRAGE_CORNER_BOTTOMLEFT)
		cairo_arc (cr, x+radius,   y+h-radius, radius, M_PI/2, M_PI);
	else
		cairo_line_to (cr, x, y+h);
	
	if (corners & MIRAGE_CORNER_TOPLEFT)
		cairo_arc (cr, x+radius,   y+radius,   radius, M_PI, 270*(M_PI/180));
	else
		cairo_line_to (cr, x, y);
}


/* KEEP IN MIND, MIRROR TAKES PLACE BEFORE ROTATION */
/* ROTATES ANTI-CLOCKWISE, I THINK :P */
/* TODO: Do I really need THREE matrices? */
static void
rotate_mirror_translate (cairo_t *cr, double radius, double x, double y,
                         boolean mirror_horizontally, boolean mirror_vertically)
{
	cairo_matrix_t matrix_rotate;
	cairo_matrix_t matrix_mirror;
	cairo_matrix_t matrix_result;
	
	double r_cos = cos(radius);
	double r_sin = sin(radius);
	
	cairo_matrix_init (&matrix_rotate, r_cos,
	                                   r_sin,
	                                   r_sin,
	                                   r_cos,
	                                   x, y);
	
	cairo_matrix_init (&matrix_mirror, mirror_horizontally ? -1 : 1,
	                                   0,
	                                   0,
	                                   mirror_vertically ? -1 : 1,
									   0, 0);

	cairo_matrix_multiply (&matrix_result, &matrix_mirror, &matrix_rotate);

	cairo_set_matrix (cr, &matrix_result);
}

void
mirage_draw_inset (cairo_t *cr, int width, int height,
                       double radius, uint8 corners)
{
	cairo_pattern_t *pattern;
	
	mirage_rounded_rectangle (cr, 0, 0, width, height, radius, corners);
	
	pattern = cairo_pattern_create_linear (0, 0, 0, height);
	cairo_pattern_add_color_stop_rgba (pattern, 0.0,   0.0, 0.0, 0.0, 0.08);
	cairo_pattern_add_color_stop_rgba (pattern, 0.2,   0.0, 0.0, 0.0, 0.06);
	cairo_pattern_add_color_stop_rgba (pattern, 0.8,   0.0, 0.0, 0.0, 0.06);
	cairo_pattern_add_color_stop_rgba (pattern, 1.0,   1.0, 1.0, 1.0, 0.5);
	cairo_set_source (cr, pattern);
	cairo_pattern_destroy (pattern);
	
	cairo_stroke (cr);
}

static void
mirage_draw_shadow (cairo_t *cr, int width, int height)
{
	cairo_set_line_width (cr, 1.0);
	
	cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.1);
	
	cairo_move_to (cr, width, 3.0);
	cairo_arc (cr, width-4.0, height-4.0, 4.0, 0, M_PI/2);
	cairo_line_to (cr, 3.0, height);

	cairo_stroke (cr);
}

void
mirage_draw_button (cairo_t *cr,
                        const mirageColors *colors,
                        const WidgetParameters *params,
                        int x, int y, int width, int height)
{
	const float RADIUS = 2.0;

	CairoColor *border = (CairoColor*)&colors->shade[6];
	CairoColor fill;
	cairo_pattern_t *pattern;

	cairo_translate       (cr, x, y);
	cairo_set_line_width  (cr, 1.0);
	
	if (params->active)
		fill = (CairoColor)colors->spot[1];
	else
		fill = (CairoColor)colors->bg[MIRAGE_STATE_NORMAL];
	
	if (params->disabled)
		shade(&fill, &fill, 0.8);
	else if (params->prelight)
		shade(&fill, &fill, 1.1);

	/* Draw button base fill */
	cairo_set_source_rgb  (cr, fill.r,
	                           fill.g,
	                           fill.b);
	cairo_rectangle       (cr, 2, 2, width-4, height-4);
	cairo_fill_preserve   (cr);
	
	/* Draw the bling */
	pattern = cairo_pattern_create_linear (0, 0, 0, height);
	
	if (!params->active) {
		cairo_pattern_add_color_stop_rgba (pattern, 0.0,   1.0, 1.0, 1.0, 0.7);
		cairo_pattern_add_color_stop_rgba (pattern, 0.5,   1.0, 1.0, 1.0, 0.4);
		cairo_pattern_add_color_stop_rgba (pattern, 0.501, 1.0, 1.0, 1.0, 0.0);
		cairo_pattern_add_color_stop_rgba (pattern, 1.0,   1.0, 1.0, 1.0, 0.5);
	} else {
		cairo_pattern_add_color_stop_rgba (pattern, 0.0,   1.0, 1.0, 1.0, 0.7);
		cairo_pattern_add_color_stop_rgba (pattern, 0.5,   1.0, 1.0, 1.0, 0.5);
		cairo_pattern_add_color_stop_rgba (pattern, 0.501, 1.0, 1.0, 1.0, 0.3);
		cairo_pattern_add_color_stop_rgba (pattern, 1.0,   1.0, 1.0, 1.0, 0.5);
	}

	cairo_set_source (cr, pattern);
	cairo_fill (cr);
	cairo_pattern_destroy (pattern);
	
	/* Translate 0.5 pixels, so we draw nice strong lines */
	cairo_translate       (cr, 0.5, 0.5);
	
	cairo_set_source_rgb (cr, border->r,
		                      border->g,
		                      border->b);
	
	mirage_rounded_rectangle (cr, 1, 1,
                                 width-3, height-3,
                                 RADIUS, params->corners);
	cairo_stroke (cr);
	
	if (!params->active)
		mirage_draw_shadow (cr, width-1, height-1);
	else
		mirage_draw_inset (cr, width-1, height-1, RADIUS, params->corners);
}

void
mirage_draw_combobox (cairo_t *cr,
                         const mirageColors *colors,
                         const WidgetParameters *params,
                         const ComboBoxParameters *combobox,
                         int x, int y, int width, int height)
{
	CairoColor highlight = (CairoColor)colors->spot[1];

	CairoColor *border = (CairoColor*)&colors->shade[6];
	CairoColor fill;
	cairo_pattern_t *pattern;

	cairo_translate       (cr, x, y);
	cairo_set_line_width  (cr, 1.0);
	
	shade(&highlight, &highlight, 0.8);
	
	if (params->active)
		fill = (CairoColor)colors->spot[1];
	else
		fill = (CairoColor)colors->bg[MIRAGE_STATE_NORMAL];
	
	if (params->disabled) {
		shade(&fill, &fill, 0.8);
		shade(&highlight, &highlight, 0.8);
	}
	else if (params->prelight)
	{
		shade(&fill, &fill, 1.1);
		shade(&highlight, &highlight, 1.1);
	}

	/* Draw button base fill */
	cairo_set_source_rgb  (cr, fill.r,
	                           fill.g,
	                           fill.b);
	cairo_rectangle       (cr, 2, 2, width-4, height-4);
	cairo_fill_preserve   (cr);
	
	/* Draw the color part */
	if (combobox->separator_pos != -1)
	{
		cairo_new_path (cr);
		/*mirage_rounded_rectangle (cr,
										combobox->separator_pos + 0.5,
										params->ythickness - 1.5,
										width - params->ythickness - combobox->separator_pos,
										height - params->ythickness - 1,
										3.0,
										MIRAGE_CORNER_TOPRIGHT | MIRAGE_CORNER_BOTTOMRIGHT);*/
		cairo_rectangle (cr,
		                 		        combobox->separator_pos - 2.5,
										params->ythickness - 1.5,
										width - params->ythickness - combobox->separator_pos + 3.0,
										height - params->ythickness - 1);
		cairo_set_source_rgb           (cr, highlight.r, highlight.g, highlight.b);
		cairo_fill (cr);
		
		cairo_rectangle       (cr, 2, 2, width-4, height-4);
	}
	
	/* Draw the bling */
	pattern = cairo_pattern_create_linear (0, 0, 0, height);
	cairo_pattern_add_color_stop_rgba (pattern, 0.0,   1.0, 1.0, 1.0, 0.7);
	cairo_pattern_add_color_stop_rgba (pattern, 0.5,   1.0, 1.0, 1.0, 0.4);
	cairo_pattern_add_color_stop_rgba (pattern, 0.501, 1.0, 1.0, 1.0, 0.0);
	cairo_pattern_add_color_stop_rgba (pattern, 1.0,   1.0, 1.0, 1.0, 0.5);

	cairo_set_source (cr, pattern);
	cairo_fill (cr);
	cairo_pattern_destroy (pattern);
	
	/* Translate 0.5 pixels, so we draw nice strong lines */
	cairo_translate       (cr, 0.5, 0.5);

	/* Draw border */
	mirage_rounded_rectangle (cr, 1, 1, width-3, height-3, 4, params->corners);
	cairo_set_source_rgb           (cr, border->r, border->g, border->b);
	cairo_stroke                   (cr);

	if (combobox->separator_pos != -1)
	{	
		cairo_move_to         (cr, combobox->separator_pos - 3, params->ythickness - 1.5);
		cairo_line_to         (cr, combobox->separator_pos - 3, height - params->ythickness + 0.5);
		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.2);
		cairo_stroke          (cr);
	
		cairo_move_to         (cr, combobox->separator_pos - 2, params->ythickness - 1.5);
		cairo_line_to         (cr, combobox->separator_pos - 2, height - params->ythickness + 0.5);
		cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.3);
		cairo_stroke          (cr);
	}
}

void
mirage_draw_entry (cairo_t *cr,
                       const mirageColors *colors,
                       const WidgetParameters *params,
                       boolean is_combo,
                       int x, int y, int width, int height)
{
	CairoColor *border = (CairoColor*)&colors->shade[6];

	cairo_translate       (cr, x, y);
	cairo_set_line_width  (cr, 1.0);
	
	/* Draw entry base fill */
	cairo_set_source_rgb  (cr, colors->base[params->state_type].r,
	                           colors->base[params->state_type].g,
	                           colors->base[params->state_type].b);
	cairo_rectangle       (cr, 2, 2, width-4, height-4);
	cairo_fill            (cr);
	
	/* Translate 0.5 pixels, so we draw nice strong lines */
	cairo_translate       (cr, 0.5, 0.5);
		
	/* Draw border */
	cairo_rectangle (cr, 1, 1, width-3, height-3);
	cairo_set_source_rgb           (cr, border->r, border->g, border->b);
	cairo_stroke                   (cr);
	
	mirage_draw_inset (cr, width-1, height-1, 0, MIRAGE_CORNER_NONE);
}

void
mirage_draw_scrollbar_stepper (cairo_t *cr,
                                  const mirageColors            *colors,
                                  const WidgetParameters           *widget,
                                  const ScrollBarParameters        *scrollbar,
                                  const ScrollBarStepperParameters *stepper,
                                  int x, int y, int width, int height)
{
	mirageCorners corners = MIRAGE_CORNER_NONE;
	CairoColor  fill = (CairoColor)colors->shade[0];
	CairoColor *border = (CairoColor*)&colors->shade[6];
	CairoColor	shade1, shade2, shade3, shade4;
	cairo_pattern_t *pattern;
	
	if (scrollbar->horizontal)
	{
		cairo_translate (cr, x, y);	
	}
	else
	{
		int tmp = height;
		rotate_mirror_translate (cr, M_PI/2, x, y, FALSE, FALSE);
		height = width;
		width = tmp;
	}
	
	cairo_set_line_width (cr, 1);
	                                     
	if (widget->prelight) {
		shade (&fill, &fill, 1.1);
	}
		
	cairo_set_line_width (cr, 1);
	
	shade (&fill, &shade1, 1.1);
	shade (&fill, &shade2, 1.05);
	shade (&fill, &shade3, 0.85);
	shade (&fill, &shade4, 1.0);
	
	pattern = cairo_pattern_create_linear (1, 1, 1, height-4);
	cairo_pattern_add_color_stop_rgb (pattern, 0,   shade1.r, shade1.g, shade1.b);
	cairo_pattern_add_color_stop_rgb (pattern, 0.5, shade2.r, shade2.g, shade2.b);
	cairo_pattern_add_color_stop_rgb (pattern, 1,   shade4.r, shade4.g, shade4.b);	
	cairo_pattern_add_color_stop_rgb (pattern, 0.5, shade3.r,  shade3.g,  shade3.b);
	cairo_rectangle (cr, 0, 0, width, height);
	cairo_set_source (cr, pattern);
	cairo_fill (cr);
	cairo_pattern_destroy (pattern);
	
	cairo_rectangle (cr, 1.5, 1.5, width-3, height-3);
	cairo_set_source_rgba (cr, 1, 1, 1, 0.6);
	cairo_stroke (cr);

	cairo_rectangle (cr, 0.5, 0.5, width-1, height-1);
	cairo_set_source_rgb           (cr, border->r, border->g, border->b);
	cairo_stroke (cr);
}

void
mirage_draw_scrollbar_slider (cairo_t *cr,
                                 const mirageColors          *colors,
                                 const WidgetParameters         *widget,
                                 const ScrollBarParameters      *scrollbar,
                                 int x, int y, int width, int height)
{
	CairoColor  fill = (CairoColor)colors->shade[6];
	CairoColor *border = (CairoColor*)&colors->shade[6];
	CairoColor  shade1, shade2, shade3, shade4;
	cairo_pattern_t *pattern;

	if (scrollbar->junction & MIRAGE_JUNCTION_BEGIN)
	{
		if (scrollbar->horizontal)
		{
			x -= 1;
			width += 1;
		}
		else
		{
			y -= 1;
			height += 1;
		}
	}
	if (scrollbar->junction & MIRAGE_JUNCTION_END)
	{
		if (scrollbar->horizontal)
			width += 1;
		else
			height += 1;
	}
		
	if (scrollbar->horizontal)
	{
		cairo_translate (cr, x, y);	
	}
	else
	{
		int tmp = height;
		rotate_mirror_translate (cr, M_PI/2, x, y, FALSE, FALSE);
		height = width;
		width = tmp;
	}
	
	if (widget->prelight) {
		shade (&fill, &fill, 1.1);
	}
		
	cairo_set_line_width (cr, 1);
	
	shade (&fill, &shade1, 1.3);
	shade (&fill, &shade2, 1.1);
	shade (&fill, &shade3, 0.65);
	shade (&fill, &shade4, 0.85);
	
	pattern = cairo_pattern_create_linear (1, 1, 1, height-4);
	cairo_pattern_add_color_stop_rgb (pattern, 0,   shade1.r, shade1.g, shade1.b);
	cairo_pattern_add_color_stop_rgb (pattern, 0.5, shade2.r, shade2.g, shade2.b);
	cairo_pattern_add_color_stop_rgb (pattern, 1,   shade4.r, shade4.g, shade4.b);	
	cairo_pattern_add_color_stop_rgb (pattern, 0.5, shade3.r,  shade3.g,  shade3.b);
	cairo_rectangle (cr, 0, 0, width, height);
	cairo_set_source (cr, pattern);
	cairo_fill (cr);
	cairo_pattern_destroy (pattern);
	
	cairo_rectangle (cr, 1.5, 1.5, width-3, height-3);
	cairo_set_source_rgba (cr, 1, 1, 1, 0.4);
	cairo_stroke (cr);

	cairo_rectangle (cr, 0.5, 0.5, width-1, height-1);
	cairo_set_source_rgb           (cr, border->r, border->g, border->b);
	cairo_stroke (cr);
}

void
mirage_draw_scrollbar_trough (cairo_t *cr,
                                  const mirageColors     *colors,
                                  const WidgetParameters    *widget,
                                  const ScrollBarParameters *scrollbar,
                                  int x, int y, int width, int height)
{
	CairoColor      *bg     = (CairoColor*)&colors->shade[1];
	CairoColor      *border = (CairoColor*)&colors->shade[6];
	CairoColor       bg_shade;
	cairo_pattern_t *pattern;
	
	shade (bg, &bg_shade, 0.8);
	
	cairo_set_line_width (cr, 1);
	//cairo_translate (cr, x, y);
	
	if (scrollbar->horizontal)
	{
		int tmp = height;
		rotate_mirror_translate (cr, M_PI/2, x, y, FALSE, FALSE);
		height = width;
		width = tmp;
	}
	else
	{
		cairo_translate (cr, x, y);	
	}

	/* Draw fill */
	/*cairo_rectangle (cr, 1, 0, width-2, height);
	cairo_set_source_rgb (cr, bg->r, bg->g, bg->b);
	cairo_fill (cr);*/

	/* Draw shadow */
	pattern = cairo_pattern_create_linear (1, 0, width-2, 0);
	cairo_pattern_add_color_stop_rgba (pattern, 0,   bg_shade.r, bg_shade.g, bg_shade.b, 0.7);
	cairo_pattern_add_color_stop_rgba (pattern, 1.0, bg->r,      bg->g,      bg->b,      0.7);	
	cairo_rectangle (cr, 1, 0, width-2, height);
	cairo_set_source (cr, pattern);
	cairo_fill (cr);
	cairo_pattern_destroy (pattern);
	
	/* Draw border */
	cairo_rectangle (cr, 0.5, 0.5, width-1, height-1);
	cairo_set_source_rgb (cr, border->r, border->g, border->b);
	cairo_stroke (cr);
}

void
mirage_draw_spinbutton (cairo_t *cr,
                            const mirageColors *colors,
                            const WidgetParameters *params,
                            int x, int y, int width, int height)
{
}

void
mirage_draw_spinbutton_down (cairo_t *cr,
                                 const mirageColors *colors,
                                 const WidgetParameters *params,
                                 int x, int y, int width, int height)
{
}

static void
mirage_scale_draw_gradient (cairo_t *cr,
                                const CairoColor *c1,
                                const CairoColor *c2,
                                const CairoColor *c3,
                                int x, int y, int width, int height,
                                boolean horizontal)
{
}

void
mirage_draw_scale_trough (cairo_t *cr,
                              const mirageColors *colors,
                              const WidgetParameters *params,
                              const SliderParameters *slider,
                              int x, int y, int width, int height)
{
}

void
mirage_draw_slider_button (cairo_t *cr,
                               const mirageColors *colors,
                               const WidgetParameters *params,
                               const SliderParameters *slider,
                               int x, int y, int width, int height)
{
}

void
mirage_draw_progressbar_trough (cairo_t *cr,
                                    const mirageColors *colors,
                                    const WidgetParameters *params,
                                    int x, int y, int width, int height)
{
}

void
mirage_draw_progressbar_fill (cairo_t *cr,
                                  const mirageColors *colors,
                                  const WidgetParameters *params,
                                  const ProgressBarParameters *progressbar,
                                  int x, int y, int width, int height)
{
}

void
mirage_draw_optionmenu (cairo_t *cr,
                            const mirageColors *colors,
                            const WidgetParameters *params,
                            const OptionMenuParameters *optionmenu,
                            int x, int y, int width, int height)
{
}

void
mirage_draw_tab (cairo_t *cr,
                    const mirageColors  *colors,
                    const WidgetParameters *params,
                    const TabParameters    *tab,
                    int x, int y, int width, int height)
{
	const float RADIUS = 3.0;
	CairoColor          *border1       = (CairoColor*)&colors->shade[6];
	CairoColor          *stripe_fill   = (CairoColor*)&colors->spot[1];
	CairoColor          *stripe_border = (CairoColor*)&colors->spot[2];
	CairoColor          *fill;

	cairo_pattern_t     *pattern;
	
	int                  corners;
	double               strip_size;
		
	/* Set clip */
	cairo_rectangle      (cr, x, y, width, height);
	cairo_clip           (cr);
	cairo_new_path       (cr);

	/* Translate and set line width */	
	cairo_set_line_width (cr, 1.0);
	cairo_translate      (cr, x+0.5, y+0.5);


	/* Make the tabs slightly bigger than they should be, to create a gap */
	/* And calculate the strip size too, while you're at it */
	if (tab->gap_side == MIRAGE_GAP_TOP || tab->gap_side == MIRAGE_GAP_BOTTOM)
	{
		height += RADIUS;
	 	strip_size = ((100.0/height)*2.0)/100.0;
		
		if (tab->gap_side == MIRAGE_GAP_TOP)
		{
			cairo_translate (cr, 0.0, -4.0); /* gap at the other side */
			corners = MIRAGE_CORNER_BOTTOMLEFT | MIRAGE_CORNER_BOTTOMRIGHT;
		}
		else
			corners = MIRAGE_CORNER_TOPLEFT | MIRAGE_CORNER_TOPRIGHT;
	}
	else
	{
		width += RADIUS;
	 	strip_size = ((100.0/width)*2.0)/100.0;
		
		if (tab->gap_side == MIRAGE_GAP_LEFT) 
		{
			cairo_translate (cr, -4.0, 0.0); /* gap at the other side */
			corners = MIRAGE_CORNER_TOPRIGHT | MIRAGE_CORNER_BOTTOMRIGHT;		
		}
		else
			corners = MIRAGE_CORNER_TOPLEFT | MIRAGE_CORNER_BOTTOMLEFT;	
	}
	
	/* Set the fill color */
	if (params->active)
		fill = (CairoColor*)&colors->bg[params->state_type];
	else
		fill = (CairoColor*)&params->parentbg;
	

	/* Set tab shape */
	mirage_rounded_rectangle (cr, 0,
	                             0,
	                             width-1,
	                             height-1,
	                             RADIUS, corners);
	
	/* Draw fill */
	/*cairo_set_source_rgb  (cr, fill->r, fill->g, fill->b);
	cairo_fill_preserve   (cr);*/
	
	/* Draw highlight */
	/*if (!params->active)
	{
		ShadowParameters shadow;
		
		shadow.shadow  = CL_SHADOW_OUT;
		shadow.corners = corners;
		
		mirage_draw_highlight_and_shade (cr, &shadow,
		                                     width,
		                                     height, RADIUS);
	}*/
	

	if (params->active)
	{
		cairo_set_line_width (cr, 1);
		cairo_set_source_rgba (cr, 1, 1, 1, 0.4);
		
		if (tab->gap_side == MIRAGE_GAP_TOP || tab->gap_side == MIRAGE_GAP_BOTTOM)
			cairo_rectangle (cr, 1, 1, width-3, height-3);
		else
			cairo_rectangle (cr, 1, 1, width-3, height-3);
		
		cairo_stroke (cr);
	}
	else
	{
		/* Draw shade */
		pattern = cairo_pattern_create_linear ( tab->gap_side == MIRAGE_GAP_LEFT   ? width-2  : 0,
		                                        tab->gap_side == MIRAGE_GAP_TOP    ? height-2 : 0,
		                                        tab->gap_side == MIRAGE_GAP_RIGHT  ? width    : 0,
		                                        tab->gap_side == MIRAGE_GAP_BOTTOM ? height   : 0 );
	
		mirage_rounded_rectangle (cr, 0,
	                                 0,
	                                 width-1,
	                                 height-1,
	                                 RADIUS, corners);
		
		cairo_pattern_add_color_stop_rgb  (pattern, 0.0,        stripe_fill->r, stripe_fill->g, stripe_fill->b);
		cairo_pattern_add_color_stop_rgb  (pattern, strip_size, stripe_fill->r, stripe_fill->g, stripe_fill->b);
		cairo_pattern_add_color_stop_rgba (pattern, strip_size, 1.0, 1.0, 1.0, 0.5);
		cairo_pattern_add_color_stop_rgba (pattern, 0.8,        1.0, 1.0, 1.0, 0.0);
		
		/*cairo_pattern_add_color_stop_rgba (pattern, 0.0, 1.0, 1.0, 1.0, 0.5);
		cairo_pattern_add_color_stop_rgba (pattern, 0.2, stripe_fill->r, stripe_fill->g, stripe_fill->b, 0.2);
		cairo_pattern_add_color_stop_rgba (pattern, 0.2, stripe_fill->r, stripe_fill->g, stripe_fill->b, 0.5);
		cairo_pattern_add_color_stop_rgba (pattern, 1.0, 1.0, 1.0, 1.0, 0.0);*/
		cairo_set_source (cr, pattern);
		cairo_fill (cr);
		cairo_pattern_destroy (pattern);
	}

	
	mirage_rounded_rectangle (cr, 0,
	                             0,
	                             width-1,
	                             height-1,
	                             RADIUS, corners);
	
	
	if (params->active)
	{
		cairo_set_source_rgb  (cr, border1->r, border1->g, border1->b);	
		cairo_stroke (cr);
	}
	else
	{
		pattern = cairo_pattern_create_linear ( tab->gap_side == MIRAGE_GAP_LEFT   ? width-2  : 2,
		                                        tab->gap_side == MIRAGE_GAP_TOP    ? height-2 : 2,
		                                        tab->gap_side == MIRAGE_GAP_RIGHT  ? width    : 2,
		                                        tab->gap_side == MIRAGE_GAP_BOTTOM ? height   : 2 );
		
		cairo_pattern_add_color_stop_rgb (pattern, 0.0,        stripe_border->r, stripe_border->g, stripe_border->b);
		cairo_pattern_add_color_stop_rgb (pattern, strip_size, stripe_border->r, stripe_border->g, stripe_border->b);
		cairo_pattern_add_color_stop_rgb (pattern, strip_size, border1->r,       border1->g,       border1->b);
		cairo_pattern_add_color_stop_rgb (pattern, 1.0,        border1->r,       border1->g,       border1->b);
		cairo_set_source (cr, pattern);
		cairo_stroke (cr);
		cairo_pattern_destroy (pattern);
	}
}

static void
mirage_get_frame_gap_clip (int x, int y, int width, int height, 
                               FrameParameters     *frame,
                               mirageRectangle *bevel,
                               mirageRectangle *border)
{
	if (frame->gap_side == MIRAGE_GAP_TOP)
	{
		MIRAGE_RECTANGLE_SET ((*bevel),  1.5 + frame->gap_x,  -0.5,
											 frame->gap_width - 3, 2.0);
		MIRAGE_RECTANGLE_SET ((*border), 0.5 + frame->gap_x,  -0.5,
											 frame->gap_width - 2, 2.0);
	}
	else if (frame->gap_side == MIRAGE_GAP_BOTTOM)
	{
		MIRAGE_RECTANGLE_SET ((*bevel),  1.5 + frame->gap_x,  height - 2.5,
											 frame->gap_width - 3, 2.0);
		MIRAGE_RECTANGLE_SET ((*border), 0.5 + frame->gap_x,  height - 1.5,
											 frame->gap_width - 2, 2.0);		
	}
	else if (frame->gap_side == MIRAGE_GAP_LEFT)
	{
		MIRAGE_RECTANGLE_SET ((*bevel),  -0.5, 1.5 + frame->gap_x,
											 2.0, frame->gap_width - 3);
		MIRAGE_RECTANGLE_SET ((*border), -0.5, 0.5 + frame->gap_x,
											 1.0, frame->gap_width - 2);			
	}
	else if (frame->gap_side == MIRAGE_GAP_RIGHT)
	{
		MIRAGE_RECTANGLE_SET ((*bevel),  width - 2.5, 1.5 + frame->gap_x,
											 2.0, frame->gap_width - 3);
		MIRAGE_RECTANGLE_SET ((*border), width - 1.5, 0.5 + frame->gap_x,
											 1.0, frame->gap_width - 2);			
	}
}

void
mirage_draw_frame (cairo_t *cr,
                      const mirageColors *colors,
                      const WidgetParameters *params,
                      const FrameParameters      *frame,
                      int x, int y, int width, int height)
{
	const float RADIUS = 3.0;
	CairoColor *border = frame->border;
	mirageRectangle bevel_clip;
	mirageRectangle frame_clip;
	
	if (frame->shadow == MIRAGE_SHADOW_NONE)
		return;
	
	if (frame->gap_x != -1)
		mirage_get_frame_gap_clip (x, y, width, height,
		                               (FrameParameters*)frame,
		                               &bevel_clip, &frame_clip);
	
	cairo_set_line_width (cr, 1.0);
	cairo_translate      (cr, x+0.5, y+0.5);
	
	/* Set clip for the bevel */
	if (frame->gap_x != -1)
	{
		/* Set clip for gap */
		cairo_set_fill_rule  (cr, CAIRO_FILL_RULE_EVEN_ODD);
		cairo_rectangle      (cr, -0.5, -0.5, width, height);
		cairo_rectangle      (cr, bevel_clip.x, bevel_clip.y, bevel_clip.width, bevel_clip.height);
		cairo_clip           (cr);
		cairo_new_path       (cr);
	}
	
	/* Draw the bevel */
	if (frame->shadow == MIRAGE_SHADOW_ETCHED_IN || frame->shadow == MIRAGE_SHADOW_ETCHED_OUT)
	{
		cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.8 );
		if (frame->shadow == MIRAGE_SHADOW_ETCHED_IN)
			cairo_rectangle       (cr, 1, 1, width-2, height-2);
		else
			cairo_rectangle       (cr, 0, 0, width-2, height-2);
		cairo_stroke          (cr);
	}
	else if (frame->shadow != MIRAGE_SHADOW_NONE)
	{
		ShadowParameters shadow;
		shadow.corners = params->corners;
		shadow.shadow  = frame->shadow;
		//mirage_draw_highlight_and_shade (cr, &shadow, width, height, RADIUS);
	}
	
	/* Set clip for the frame */
	cairo_reset_clip     (cr);
	if (frame->gap_x != -1)
	{
		/* Set clip for gap */
		cairo_set_fill_rule  (cr, CAIRO_FILL_RULE_EVEN_ODD);
		cairo_rectangle      (cr, -0.5, -0.5, width, height);
		cairo_rectangle      (cr, frame_clip.x, frame_clip.y, frame_clip.width, frame_clip.height);
		cairo_clip           (cr);
		cairo_new_path       (cr);
	}

	/* Draw frame */
	if (frame->shadow == MIRAGE_SHADOW_ETCHED_IN || frame->shadow == MIRAGE_SHADOW_ETCHED_OUT)
	{
		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.2 );
		if (frame->shadow == MIRAGE_SHADOW_ETCHED_IN)
			cairo_rectangle       (cr, 0, 0, width-2, height-2);
		else
			cairo_rectangle       (cr, 1, 1, width-2, height-2);
	}
	else
	{
		cairo_set_source_rgb (cr, border->r, border->g, border->b );
		cairo_rectangle      (cr, 0, 0, width-1, height-1);
	}
	cairo_stroke         (cr);
}

void
mirage_draw_statusbar (cairo_t *cr,
                          const mirageColors *colors,
                          const WidgetParameters *params,
                          int x, int y, int width, int height)
{
	CairoColor *border = (CairoColor*)&colors->shade[6];
	
	cairo_translate       (cr, x + 0.5, y + 0.5);
	cairo_set_line_width  (cr, 1.0);
	
	/* Draw line */
	cairo_move_to         (cr, 0, 0);
	cairo_line_to         (cr, width-1, 0);
	cairo_set_source_rgb  (cr, border->r, border->g, border->b);
	cairo_stroke          (cr);	

	/* Draw highlight */
	cairo_move_to         (cr, 0, 1);
	cairo_line_to         (cr, width-1, 1);
	cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.2);
	cairo_stroke          (cr);	
}
