#include <gtk/gtk.h>
#include <cairo.h>

#include "mirage_style.h"
#include "mirage_rc_style.h"
#include "mirage_draw.h"

#include <math.h>
#include <string.h>

#include "bits.c"
#include "support.h"

/* #define DEBUG 1 */

#define DETAIL(xx)          ((detail) && (!strcmp(xx, detail)))

#define DRAW_ARGS    GtkStyle       *style, \
                     GdkWindow      *window, \
                     GtkStateType    state_type, \
                     GtkShadowType   shadow_type, \
                     GdkRectangle   *area, \
                     GtkWidget      *widget, \
                     const gchar    *detail, \
                     gint            x, \
                     gint            y, \
                     gint            width, \
                     gint            height

static GdkGC *realize_color (GtkStyle * style, GdkColor * color);
static GtkStyleClass *parent_class;

static void
print_widget_path (GtkWidget *widget)
{
	char *path;
	
	gtk_widget_path (widget, NULL, &path, NULL);
	
	printf("%s\n", path);
}

static cairo_t *
mirage_begin_paint (GdkDrawable  *window, GdkRectangle *area)
{
    cairo_t *cr;

    cr = (cairo_t*)gdk_cairo_create (window);

    if (area) {
        cairo_rectangle (cr, area->x, area->y, area->width, area->height);
        cairo_clip (cr);
        cairo_new_path (cr);
    }

    return cr;
}

static void
mirage_set_widget_parameters (const GtkWidget      *widget,
                                  const GtkStyle       *style,
                                  GtkStateType          state_type,
                                  WidgetParameters     *params)
{
		if (widget && GTK_IS_ENTRY (widget))
			state_type = GTK_WIDGET_STATE (widget);

		params->active     = (state_type == GTK_STATE_ACTIVE);
		params->prelight   = (state_type == GTK_STATE_PRELIGHT);
		params->disabled   = (state_type == GTK_STATE_INSENSITIVE);			
		params->state_type = (mirageStateType)state_type;
		params->corners     = MIRAGE_CORNER_ALL;
		
		if (widget && GTK_IS_TOGGLE_BUTTON (widget))
			params->active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
		
		params->xthickness = style->xthickness;
		params->ythickness = style->ythickness;		
		
		/* I want to avoid to have to do this. I need it for GtkEntry, unless I
		   find out why it doesn't behave the way I expect it to. */
		get_parent_bg (widget, &params->parentbg);		
}

static void
draw_flat_box (DRAW_ARGS)
{
	MirageStyle  *mirage_style = MIRAGE_STYLE (style);
	mirageColors *colors = &mirage_style->colors;
	
	cairo_t *cr;

	cr     = mirage_begin_paint (window, area);
	colors = &mirage_style->colors;
	
	if ((width == -1) && (height == -1))
		gdk_window_get_size (window, &width, &height);
	else if (width == -1)
		gdk_window_get_size (window, &width, NULL);
	else if (height == -1)
		gdk_window_get_size (window, NULL, &height);
	
	printf( "draw_flat_box: %s %s\n", detail, G_OBJECT_TYPE_NAME (widget));
	if (DETAIL ("notebook"))
	{
		parent_class->draw_flat_box (style, window, state_type, shadow_type,
								     area, widget, detail,
								     x, y, width, height);	
	}
	if (DETAIL ("base"))
	{
		cairo_pattern_t *pattern;
		CairoColor       center;
		
		shade (&colors->bg[0], &center, 1.2);

		pattern = cairo_pattern_create_linear (0, 0, width, 0);
		cairo_pattern_add_color_stop_rgb (pattern, 0.0, colors->bg[0].r,
		                                                colors->bg[0].g,
		                                                colors->bg[0].b);
		cairo_pattern_add_color_stop_rgb (pattern, 0.5, center.r,
		                                                center.g,
		                                                center.b);		
		cairo_pattern_add_color_stop_rgb (pattern, 1.0, colors->bg[0].r,
		                                                colors->bg[0].g,
		                                                colors->bg[0].b);
		
		cairo_set_source (cr, pattern);
		cairo_rectangle  (cr, 0, 0, width, height);
		cairo_fill       (cr);
		
		cairo_pattern_destroy (pattern);
		cairo_destroy (cr);
	}
	else if (detail && 	
	    state_type == GTK_STATE_SELECTED && (
	    !strncmp ("cell_even", detail, strlen ("cell_even")) ||
	    !strncmp ("cell_odd", detail, strlen ("cell_odd"))))
	{
		cairo_pattern_t *pattern;
		CairoColor       lower_color;
		CairoColor       upper_color;
		
		cairo_translate (cr, x, y);

		if (GTK_WIDGET_HAS_FOCUS (widget))
		{
			upper_color = colors->base[state_type];
			shade (&upper_color, &lower_color, 0.95);
			//shade (&upper_color, &upper_color, 1.00);
		}
		else
		{
			upper_color = colors->base[GTK_STATE_ACTIVE];
			shade (&upper_color, &lower_color, 0.97);
			//shade (&upper_color, &upper_color, 1.00);
		}
			
		pattern = cairo_pattern_create_linear (0, 0, 0, height);
		cairo_pattern_add_color_stop_rgb (pattern, 0.0, upper_color.r,
		                                                upper_color.g,
		                                                upper_color.b);
		cairo_pattern_add_color_stop_rgb (pattern, 1.0, lower_color.r,
		                                                lower_color.g,
		                                                lower_color.b);
		
		cairo_set_source (cr, pattern);
		cairo_rectangle  (cr, 0, 0, width, height);
		cairo_fill       (cr);
		
		cairo_pattern_destroy (pattern);
		cairo_destroy (cr);
	}
	else
	{
		printf( "draw_flat_box: %s %s\n", detail, G_OBJECT_TYPE_NAME (widget));
		parent_class->draw_flat_box (style, window, state_type, shadow_type,
								   area, widget, detail,
								   x, y, width, height);
	}
}

static void
draw_shadow (DRAW_ARGS)
{
	MirageStyle *mirage_style = MIRAGE_STYLE (style);
	const mirageColors *colors;
	cairo_t *cr;
	boolean combo = FALSE;

	cr     = mirage_begin_paint (window, area);
	colors = &mirage_style->colors;

	if (DETAIL ("entry"))
	{
		WidgetParameters params;
		
		mirage_set_widget_parameters (widget, style, state_type, &params);
		
		params.corners = MIRAGE_CORNER_NONE;
		if (widget && (GTK_IS_COMBO_BOX_ENTRY (widget->parent) || GTK_IS_COMBO (widget->parent)))
		{
			width += style->xthickness;
			combo = TRUE;
		}
		
		mirage_draw_entry (cr, &mirage_style->colors, &params, combo,
		                      x, y, width, height);
	}
	else if (DETAIL ("frame") && widget && GTK_IS_STATUSBAR (widget->parent))
	{
		mirage_draw_statusbar (cr, colors, NULL, x, y, width, height);
	}
	else if (DETAIL ("scrolled_window"))
	{
		CairoColor *border = (CairoColor*)&colors->shade[6];
		
		cairo_translate       (cr, x + 0.5, y + 0.5);
		cairo_set_line_width  (cr, 1.0);
		
		/* Draw rectangle */
		cairo_rectangle       (cr, 1, 1, width-3, height-3);
		cairo_set_source_rgb  (cr, border->r, border->g, border->b);
		cairo_stroke          (cr);
		
		/* Draw bottom highlight */
		cairo_move_to         (cr, 1, height-1);
		cairo_line_to         (cr, width-2, height-1);
		cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.5);
		cairo_stroke          (cr);
	}
	else
	{
		printf("draw_shadow: %s %s\n", detail, G_OBJECT_TYPE_NAME (widget));
		/*parent_class->draw_shadow (style, window, state_type, shadow_type,
								   area, widget, detail,
								   x, y, width, height);*/
	}
	cairo_destroy (cr);
}

static void 
draw_box_gap (DRAW_ARGS,
	      GtkPositionType gap_side,
	      gint            gap_x,
	      gint            gap_width)
{
	MirageStyle  *mirage_style = MIRAGE_STYLE (style);
	mirageColors *colors = &mirage_style->colors;
	cairo_t          *cr;
	
	cr = mirage_begin_paint (window, area);

	if (DETAIL ("notebook"))
	{
		WidgetParameters params;
		FrameParameters  frame;
		
		frame.shadow    = shadow_type;
		frame.gap_side  = gap_side;
		frame.gap_x     = gap_x;
		frame.gap_width = gap_width;
		frame.border    = &colors->shade[6];
		
		mirage_set_widget_parameters (widget, style, state_type, &params);

		params.corners = MIRAGE_CORNER_NONE;
		
		mirage_draw_frame (cr, colors, &params, &frame,
		                       x, y, width, height);
	}
	else
	{
		if (widget)
			printf("box_gap: %s %s\n", detail, G_OBJECT_TYPE_NAME (widget));
		parent_class->draw_box_gap (style, window, state_type, shadow_type,
									   area, widget, detail,
									   x, y, width, height,
									   gap_side, gap_x, gap_width);
	}
	
	cairo_destroy (cr);	
}

static void
draw_extension (DRAW_ARGS, GtkPositionType gap_side)
{
	MirageStyle  *mirage_style = MIRAGE_STYLE(style);
	mirageColors *colors = &mirage_style->colors;
	cairo_t         *cr;
	
	cr = mirage_begin_paint (window, area);
	
	if (DETAIL ("tab"))
	{
		WidgetParameters params;
		TabParameters    tab;
		
		mirage_set_widget_parameters (widget, style, state_type, &params);
		
		tab.gap_side = (mirageGapSide)gap_side;
		
		if (gap_side == GTK_POS_BOTTOM)
			params.corners = MIRAGE_CORNER_TOPLEFT | MIRAGE_CORNER_TOPRIGHT;
		else if (gap_side == GTK_POS_TOP)
			params.corners = MIRAGE_CORNER_BOTTOMLEFT | MIRAGE_CORNER_BOTTOMRIGHT;
			
		mirage_draw_tab (cr, colors, &params, &tab,
		                    x, y, width, height);
	}
	else
	{
		prinft("draw_extension: %s\n", detail);
		parent_class->draw_extension (style, window, state_type, shadow_type, area,
		                              widget, detail, x, y, width, height,
		                              gap_side);
	}
}
    

static void 
draw_handle (DRAW_ARGS, GtkOrientation orientation)
{
	parent_class->draw_handle (style, window, state_type, shadow_type, area,
	                           widget, detail, x, y, width, height,
	                           orientation);
}

static int
combo_box_get_seperator_pos (GtkWidget *widget)
{
	GList *children;
	int pos = 0;
	
	if (widget && GTK_IS_COMBO_BOX_ENTRY (widget->parent))
		return -1;
	
	if (widget && GTK_IS_COMBO_BOX (widget->parent))
	{
		children = gtk_container_get_children (GTK_CONTAINER (widget));
		
		if (children && GTK_IS_HBOX (children->data))
			children = gtk_container_get_children (GTK_CONTAINER (children->data));

		if (children)
		{
			do
			{
				if (GTK_IS_ARROW (children->data) || GTK_IS_VSEPARATOR (children->data))
					pos += GTK_WIDGET (children->data)->allocation.width;
				
			} while (children = g_list_next (children));
		}
	}
	else
		return -1;
	
	pos += widget->style->xthickness - 1; // don't ask me why....
	
	return pos;
}

static void
draw_box (DRAW_ARGS)
{
	MirageStyle *mirage_style = MIRAGE_STYLE (style);
	const mirageColors *colors;
	cairo_t *cr;

	cr     = mirage_begin_paint (window, area);
	colors = &mirage_style->colors;

	printf("draw_box: %s %s\n", detail, G_OBJECT_TYPE_NAME (widget));
	
	if (DETAIL ("button") && widget && GTK_IS_COMBO_BOX (widget->parent))
	{
		WidgetParameters params;
		ComboBoxParameters combobox;
		int pos;
		
		pos = combo_box_get_seperator_pos (widget);
		
		if (pos == -1)
		{
			combobox.separator_pos = style->xthickness;
			combobox.highlight     = FALSE;
		}
		else
		{
			combobox.separator_pos = width - 3 - pos;
			combobox.highlight     = TRUE;
		}
		
		mirage_set_widget_parameters (widget, style, state_type, &params);
		
		if (GTK_IS_COMBO_BOX_ENTRY (widget->parent))
		{
			params.corners   = MIRAGE_CORNER_NONE;
			
			x     -= style->xthickness;
			width += style->xthickness;
		}
		
		mirage_draw_combobox (cr, colors, &params, &combobox,
		                         x, y, width, height);
		
	}
	else if (DETAIL ("button") || DETAIL ("buttondefault"))
	{
		WidgetParameters params;
		
		mirage_set_widget_parameters (widget, style, state_type, &params);
		
		mirage_draw_button (cr, &mirage_style->colors, &params,
		                       x, y, width, height);
	}
	else if (DETAIL ("menubar"))
	{
		cairo_pattern_t *pattern;
		CairoColor highlight = (CairoColor)colors->spot[1];
		
		cairo_translate (cr, x, y);
		cairo_rectangle (cr, 0, 0, width, height);
		
		shade(&highlight, &highlight, 0.8);
		
		pattern = cairo_pattern_create_linear (0, 0, width, 0);
		cairo_pattern_add_color_stop_rgb (pattern, 0.0, colors->bg[0].r, colors->bg[0].g, colors->bg[0].b);
		cairo_pattern_add_color_stop_rgb (pattern, 0.5, highlight.r, highlight.g, highlight.b);
		cairo_pattern_add_color_stop_rgb (pattern, 1.0, colors->bg[0].r, colors->bg[0].g, colors->bg[0].b);
		cairo_set_source (cr, pattern);
		cairo_fill_preserve (cr);
		cairo_pattern_destroy (pattern);

		pattern = cairo_pattern_create_linear (0, 0, 0, height);
		cairo_pattern_add_color_stop_rgba (pattern, 0.0,   1.0, 1.0, 1.0, 0.2);
		cairo_pattern_add_color_stop_rgba (pattern, 0.4,   1.0, 1.0, 1.0, 0.10);
		cairo_pattern_add_color_stop_rgba (pattern, 0.401, 1.0, 1.0, 1.0, 0.0);
		cairo_pattern_add_color_stop_rgba (pattern, 1.0,   1.0, 1.0, 1.0, 0.05);		
		cairo_set_source (cr, pattern);
		cairo_fill (cr);
		
		cairo_pattern_destroy (pattern);
		
		cairo_rectangle (cr, 1, 1, width-2, height-2);
		cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.2);
		cairo_stroke (cr);
		
		cairo_rectangle (cr, 0, 0, width, height);
		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.4);
		cairo_stroke (cr);
	}
	else if (DETAIL ("menuitem"))
	{	
		if (widget && GTK_IS_MENU_BAR (widget->parent))
		{
			cairo_translate      (cr, x+0.5, y+0.5);
			cairo_set_line_width (cr, 1.0);
	
			cairo_rectangle (cr, 0, 1, width-1, height-2);
			cairo_set_source_rgba (cr, 1, 1, 1, 0.25);
			cairo_fill (cr);
		}
	}
	else if (DETAIL ("menu"))
	{
		cairo_translate      (cr, x+0.5, y+0.5);
		cairo_set_line_width (cr, 1.0);

		cairo_rectangle (cr, 0, 0, width-1, height-1);
		cairo_set_source_rgb (cr, 0, 0, 0);
		cairo_stroke (cr);
	}
	else if (DETAIL ("toolbar") || DETAIL ("handlebox_bin") || DETAIL ("dockitem_bin")) {
	}
	else if (DETAIL ("hscrollbar") || DETAIL ("vscrollbar") || DETAIL ("slider") || DETAIL ("stepper"))
	{
		WidgetParameters    params;
		ScrollBarParameters scrollbar;
		
		mirage_set_widget_parameters (widget, style, state_type, &params);
		params.corners = MIRAGE_CORNER_NONE;
		
		scrollbar.has_color  = FALSE;
		scrollbar.horizontal = TRUE;
		scrollbar.junction   = scrollbar_get_junction (widget);
		scrollbar.steppers   = scrollbar_visible_steppers (widget);
		
		if (GTK_IS_RANGE (widget))
			scrollbar.horizontal = GTK_RANGE (widget)->orientation == GTK_ORIENTATION_HORIZONTAL;
		
		if (DETAIL ("slider"))
		{
			mirage_draw_scrollbar_slider (cr, colors, &params, &scrollbar,
			                                 x, y, width, height);
		}
		else
		{
			ScrollBarStepperParameters stepper;
			GdkRectangle this_rectangle = { x, y, width, height };

			stepper.stepper = scrollbar_get_stepper (widget, &this_rectangle);

			mirage_draw_scrollbar_stepper (cr, colors, &params, &scrollbar, &stepper,
			                                   x, y, width, height);
		}
	}
	else if (DETAIL ("trough") && widget && (GTK_IS_VSCROLLBAR (widget) || GTK_IS_HSCROLLBAR (widget)))
	{
		WidgetParameters params;
		ScrollBarParameters scrollbar;
		
		mirage_set_widget_parameters (widget, style, state_type, &params);
		params.corners = MIRAGE_CORNER_NONE;
		
		scrollbar.horizontal = TRUE;
		//scrollbar.junction   = scrollbar_get_junction (widget);
		//scrollbar.steppers   = scrollbar_visible_steppers (widget);
		
		if (GTK_IS_RANGE (widget))
			scrollbar.horizontal = GTK_RANGE (widget)->orientation == GTK_ORIENTATION_HORIZONTAL;
		
		if (scrollbar.horizontal)
		{
			x += 2;
			width -= 4;
		}
		else
		{
			y += 2;
			height -= 4;
		}
		
		mirage_draw_scrollbar_trough (cr, colors, &params, &scrollbar,
		                                  x, y, width, height);
	}
	else
	{
		parent_class->draw_box (style, window, state_type, shadow_type, area,
								widget, detail, x, y, width, height);
	}
	cairo_destroy (cr);
}

static void
draw_slider (DRAW_ARGS, GtkOrientation orientation)
{
	parent_class->draw_slider (style, window, state_type, shadow_type, area,
	                           widget, detail, x, y, width, height, orientation);
}

static void
draw_option (DRAW_ARGS)
{
	parent_class->draw_option (style, window, state_type, shadow_type, area,
	                           widget, detail, x, y, width, height);
}

/**************************************************************************/

static void 
draw_shadow_gap (DRAW_ARGS,
                 GtkPositionType gap_side,
                 gint            gap_x,
                 gint            gap_width)
{
	parent_class->draw_shadow_gap (style, window, state_type, shadow_type, area,
	                               widget, detail, x, y, width, height,
	                               gap_side, gap_x, gap_width);
}




static void
mirage_style_init_from_rc (GtkStyle * style,
			       GtkRcStyle * rc_style)
{
	MirageStyle *mirage_style = MIRAGE_STYLE (style);
	double shades[] = {1.065, 0.93, 0.896, 0.85, 0.768, 0.665, 0.4, 0.205};
	CairoColor spot_color;
	CairoColor bg_normal;
	double contrast;
	int i;
	
	bg_normal.r = style->bg[GTK_STATE_NORMAL].red   / 65535.0;
	bg_normal.g = style->bg[GTK_STATE_NORMAL].green / 65535.0;
	bg_normal.b = style->bg[GTK_STATE_NORMAL].blue  / 65535.0;

	parent_class->init_from_rc (style, rc_style);
	
	contrast = MIRAGE_RC_STYLE (rc_style)->contrast;
	
	mirage_style->progressbarstyle  = MIRAGE_RC_STYLE (rc_style)->progressbarstyle;
	mirage_style->menubarstyle      = MIRAGE_RC_STYLE (rc_style)->menubarstyle;
	mirage_style->menuitemstyle     = MIRAGE_RC_STYLE (rc_style)->menuitemstyle;
	mirage_style->listviewitemstyle = MIRAGE_RC_STYLE (rc_style)->listviewitemstyle;
	
	/* Lighter to darker */
	for (i = 0; i < 8; i++)
	{
		shade (&bg_normal, &mirage_style->colors.shade[i],
		       (shades[i]-0.7) * contrast + 0.7);
	}
		
	spot_color.r = style->bg[GTK_STATE_SELECTED].red   / 65535.0;
	spot_color.g = style->bg[GTK_STATE_SELECTED].green / 65535.0;
	spot_color.b = style->bg[GTK_STATE_SELECTED].blue  / 65535.0;
	
	shade (&spot_color, &mirage_style->colors.spot[0], 1.42);
	shade (&spot_color, &mirage_style->colors.spot[1], 1.05);
	shade (&spot_color, &mirage_style->colors.spot[2], 0.65);
	
	for (i=0; i<5; i++)
	{
		gdk_color_to_rgb (&style->bg[i],
		                  &mirage_style->colors.bg[i].r,
		                  &mirage_style->colors.bg[i].g,
		                  &mirage_style->colors.bg[i].b);

		gdk_color_to_rgb (&style->base[i],
		                  &mirage_style->colors.base[i].r,
		                  &mirage_style->colors.base[i].g,
		                  &mirage_style->colors.base[i].b);
	}
}

static GdkGC *
realize_color (GtkStyle * style,
	       GdkColor * color)
{
	GdkGCValues gc_values;
	gdk_colormap_alloc_color (style->colormap, color, FALSE, TRUE);
	gc_values.foreground = *color;
	return gtk_gc_get (style->depth, style->colormap, &gc_values, GDK_GC_FOREGROUND);
}


static void
draw_focus (GtkStyle *style, GdkWindow *window, GtkStateType state_type,
            GdkRectangle *area, GtkWidget *widget, const gchar *detail,
            gint x, gint y, gint width, gint height)
{
	return;
	parent_class->draw_focus (style, window, state_type, area,
	                          widget, detail, x, y, width, height);
}

static gboolean
label_is_on_button (GtkWidget *label)
{
	GtkWidget *parent;
	
	g_return_val_if_fail (label != NULL, FALSE);
	
	parent = label->parent;

	while (parent)
	{
		if ((GTK_IS_BUTTON (parent) && !GTK_IS_CHECK_BUTTON (parent)) || GTK_IS_COMBO_BOX (parent))
			return TRUE;
		
		parent = parent->parent;
	}
	
	return FALSE;
}

static gboolean
label_is_on_menu_bar (GtkWidget *label)
{
	return (label != NULL) && label->parent && GTK_IS_MENU_BAR (label->parent->parent);
}

static void
draw_layout  (GtkStyle  *style,
              GdkWindow              *window,
              GtkStateType            state_type,
              gboolean                use_text,
              GdkRectangle           *area,
              GtkWidget              *widget,
              const gchar            *detail,
              gint                    x,
              gint                    y,
              PangoLayout            *layout)
{
	MirageStyle *mirage_style = MIRAGE_STYLE (style);
//	printf("%s %s\n", detail, G_OBJECT_TYPE_NAME (widget));
	
	if (DETAIL ("label") && widget && state_type != GTK_STATE_INSENSITIVE)
	{
		CairoColor tmp;
		GdkColor shadowcolor;;
		GdkGC *gc;
		GtkWidget *parent;
		
		if (GTK_IS_MENU_ITEM (widget->parent))
			parent = widget->parent->parent;
		else
			parent = widget->parent;
		
		/*gchar *blah;
		gtk_widget_class_path (widget, NULL, &blah, NULL);
			printf("%s\n", blah);*/
		
		gdk_color_to_rgb ( &parent->style->bg[0], 
		                   &tmp.r, &tmp.g, &tmp.b);
		
		
		gc = use_text ? style->text_gc[state_type] : style->fg_gc[state_type];
		
		if (DETAIL ("cellrenderertext") ||
		    label_is_on_menu_bar (widget) ||
		    label_is_on_button (widget))
		{
			shade (&tmp, &tmp, 0.8);
			cairo_color_to_gdk_color ( &tmp, &shadowcolor);
			gdk_draw_layout_with_colors (window, gc, x+1, y+1, layout, &shadowcolor, NULL);
		}
		/*else
		{
			tmp.r = 0.8;
			tmp.g = 0.8;
			tmp.b = 0.8;
			cairo_color_to_gdk_color (&tmp, &shadowcolor);
		}*/
		
		gdk_draw_layout (window, gc, x,   y,   layout);
	}
	else
	{
		parent_class->draw_layout (style, window, state_type, use_text, area,
		                           widget,detail, x, y, layout);
	}

}

static void
draw_vline                      (GtkStyle               *style,
                                 GdkWindow              *window,
                                 GtkStateType            state_type,
                                 GdkRectangle           *area,
                                 GtkWidget              *widget,
                                 const gchar            *detail,
                                 gint                    y1,
                                 gint                    y2,
                                 gint                    x)
{
	
}

static void
mirage_style_unrealize (GtkStyle * style)
{	
	parent_class->unrealize (style);
}

static void
mirage_style_init (MirageStyle * style)
{
}

static void
mirage_style_class_init (MirageStyleClass * klass)
{
	GtkStyleClass *style_class = GTK_STYLE_CLASS (klass);
	
	parent_class = g_type_class_peek_parent (klass);

	style_class->unrealize       = mirage_style_unrealize;
	style_class->init_from_rc    = mirage_style_init_from_rc;
	style_class->draw_handle     = draw_handle;
	style_class->draw_slider     = draw_slider;
	style_class->draw_shadow_gap = draw_shadow_gap;
	style_class->draw_focus      = draw_focus;
	style_class->draw_box        = draw_box;
	style_class->draw_shadow     = draw_shadow;
	style_class->draw_box_gap    = draw_box_gap;
	style_class->draw_extension  = draw_extension;
	style_class->draw_option     = draw_option;
	style_class->draw_flat_box   = draw_flat_box;
	style_class->draw_layout     = draw_layout;
	style_class->draw_vline      = draw_vline;
}

GType mirage_type_style = 0;

void
mirage_style_register_type (GTypeModule * module)
{
	static const GTypeInfo object_info =
	{
		sizeof (MirageStyleClass),
		(GBaseInitFunc) NULL,
		(GBaseFinalizeFunc) NULL,
		(GClassInitFunc) mirage_style_class_init,
		NULL,         /* class_finalize */
		NULL,         /* class_data */
		sizeof (MirageStyle),
		0,            /* n_preallocs */
		(GInstanceInitFunc) mirage_style_init,
		NULL
	};

	mirage_type_style = g_type_module_register_type (module,
	                                                     GTK_TYPE_STYLE,
	                                                     "MirageStyle",
	                                                     &object_info, 0);
}
